@@ -3,6 +3,7 @@ from .chain import Chain
 from .state import State
 from .system import System
 from .timestamp import Timestamp
+from .utility import Utility
 
 __all__ = [
     "Author",
@@ -10,4 +11,5 @@ __all__ = [
     "State",
     "System",
     "Timestamp",
+    "Utility",
 ]
